/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "pidq.h"
#include "vector.h"
#include "fp_math.h"
#include "svpwm.h"
#include "control.h"
#include "calib.h"
#include "pu.h"
#include "init.h"

struct CONTROL_VARS ControlVars;
struct ANALOG_INPUTS AIn;
struct ADC_OPROBES AdcOvrProbes;

// fixed-point multiplication
#define Q_MUL(A,B,Q) (int16_t)(((int32_t)((int16_t)(A))*(int32_t)((int16_t)(B)))>>(Q))
// fixed-point constant
#define INV_SQRT3_Q15 (int16_t)(1.0 / 1.73 * 32768.0)

// Plexim control library objects
PLX_PIDQ_Obj_t PiDqObj;
PLX_PIDQ_Handle_t PiDqHandle;

PLX_SVPWM_Obj_t SvpwmObj;
PLX_SVPWM_Handle_t SvpwmHandle;

void InitControl(void)
{
    uint16_t pwmPeriod = 1999;   // F_pwm = Fcy / [(PxTPER+1)*2*1] = 40MHz / (2000*2) = 10KHz
    InitPWM(pwmPeriod);
    InitADC();
    InitDeadTimeModule();

    ControlVars.PSECMP = P1SECMP;

    ControlVars.task1Ctr = 0;
    InitPuVars(10000); // task 1 frequency
    
    PiDqHandle = PLX_PIDQ_init(&PiDqObj,sizeof(PiDqObj));

    SvpwmHandle = PLX_SVPWM_init(&SvpwmObj,sizeof(SvpwmObj));
    PLX_SVPWM_configure(SvpwmHandle, pwmPeriod, 2);

    DisableActuation();
    ResetControl();
}

void DisableActuation(void)
{
}

void EnableActuation(void)
{
    ResetControl();
    P1TCONbits.PTEN = 0;   // Disabling the PWM timer
}

void ResetControl(void)
{
    PLX_PIDQ_configure(PiDqHandle, PuVars.wrefTsScaling, Calib.KpD, Calib.KiD, Calib.KpQ, Calib.KiQ);
    PLX_PIDQ_reset(PiDqHandle);

    ControlVars.CmpU = 0;
    ControlVars.CmpV = 0;
    ControlVars.CmpW = 0;
    ControlVars.task1Ctr = 0;
    P1DC1=0;
    ControlVars.PDC1 = ControlVars.CmpU << 1;
    ControlVars.PDC2 = ControlVars.CmpV << 1;
    ControlVars.PDC3 = ControlVars.CmpW << 1;
}

void ControlTask1(void) {
    ControlVars.task1Ctr++;

    ControlVars.KpD = Calib.KpD;
    ControlVars.fCalib = Calib.fCalib;

    // analog inputs
    SET_OPROBE(AdcOvrProbes.ADCRESULT0, 0);
    SET_OPROBE(AdcOvrProbes.ADCRESULT1, 0);
    SET_OPROBE(AdcOvrProbes.ADCRESULT2, 0);

    // process current measurements
    int16_t iu, iv;

    iu = AdcOvrProbes.ADCRESULT0 - AIn.IuOffset;
    iu = Q_MUL(iu, Calib.currentGain, Q_IV);
    iv = AdcOvrProbes.ADCRESULT1 - AIn.IvOffset;
    iv = Q_MUL(iv, Calib.currentGain, Q_IV);

    SET_OPROBE(AIn.Iu, iu);
    SET_OPROBE(AIn.Iv, iv);
    SET_OPROBE(AIn.Iw, -(iu + iv));

    // process DC voltage measurement
    int16_t vdc =  Q_MUL(AdcOvrProbes.ADCRESULT2, Calib.voltageGain, Q_IV);
    SET_OPROBE(AIn.Vdc, vdc);

    SET_OPROBE(ControlVars.fluxPosition, 0);

    // set-points
    SET_OPROBE(ControlVars.IdSet, 0);
    SET_OPROBE(ControlVars.IqSet, 0);

    // calculate maximal achievable phase voltage based on DC-link voltage (max is 1/sqrt(3))
    int16_t vsmax = Q_MUL(AIn.Vdc, INV_SQRT3_Q15, 15);
    // apply margin for current regulator
    vsmax = Q_MUL(vsmax, (int16_t) (0.95 * 32768.0), 15);
    SET_OPROBE(ControlVars.Vsmax, vsmax);

    // clark transformation
    int16_t ia, ib;
    ia = AIn.Iu; // (2*AIn.Iu - AIn.Iv - AIn.Iw)/3
    ib = AIn.Iv - AIn.Iw;
    ib = Q_MUL(ib, INV_SQRT3_Q15, 15);

    SET_OPROBE(ControlVars.Ia, ia);
    SET_OPROBE(ControlVars.Ib, ib);

    // transform currents into rotating frame
    int16_t id, iq;
    PLX_FPMATH_lookupSinAndCos(ControlVars.fluxPosition, &ControlVars.fluxPosSin, &ControlVars.fluxPosCos);
    PLX_VECT_parkRot(ControlVars.Ia, ControlVars.Ib, &id, &iq, ControlVars.fluxPosSin, ControlVars.fluxPosCos);
    SET_OPROBE(ControlVars.Id, id);
    SET_OPROBE(ControlVars.Iq, iq);

    // determine errors
    SET_OPROBE(ControlVars.IdErr, ControlVars.IdSet - ControlVars.Id);
    SET_OPROBE(ControlVars.IqErr, ControlVars.IqSet - ControlVars.Iq);

    // update decoupling gains of current regulator (could happen at a lower rate, e.g. 1000 Hz)
    SET_OPROBE(ControlVars.we, 0);
    PLX_PIDQ_updateCrossCouplingGains(PiDqHandle, ControlVars.we);
    PLX_PIDQ_applyCrossCouplingGains(PiDqHandle); // this function must not be interrupted!

    // calculate regulator output
    PLX_PIDQ_calculateOutput(PiDqHandle, ControlVars.IdErr, ControlVars.IqErr, &ControlVars.Vd, &ControlVars.Vq);

    // saturate output based on maximal allowable phase voltage
    int16_t vd = ControlVars.Vd;
    int16_t vq = ControlVars.Vq;
    PLX_VECT_satVector(ControlVars.Vsmax, &vd, &vq);

    // transform voltages back into stationary frame
    int16_t va, vb;
    PLX_VECT_invParkRot(vd, vq, &va, &vb, ControlVars.fluxPosSin, ControlVars.fluxPosCos);
    SET_OPROBE(ControlVars.Va, va);
    SET_OPROBE(ControlVars.Vb, vb);

    // determine SVPWM compare values
    int16_t svpwmLimitingGain;
    PLX_SVPWM_calcCompareValues(SvpwmHandle,
        ControlVars.Va, ControlVars.Vb, AIn.Vdc,
	&ControlVars.CmpU, &ControlVars.CmpV, &ControlVars.CmpW, &svpwmLimitingGain);

    // Setting PDC values
    ControlVars.PDC1 = ControlVars.CmpU << 1;
    ControlVars.PDC2 = ControlVars.CmpV << 1;
    ControlVars.PDC3 = ControlVars.CmpW << 1;

    // apply SVPWM saturation to regulator outputs
    SET_OPROBE(ControlVars.svpwmLimitingGain, svpwmLimitingGain);
    ControlVars.VdSat = Q_MUL(vd, ControlVars.svpwmLimitingGain, 14);
    ControlVars.VqSat = Q_MUL(vq, ControlVars.svpwmLimitingGain, 14);

    // update regulator integrator based on saturated regulator outputs
    PLX_PIDQ_updateIntegrator(PiDqHandle, ControlVars.VdSat, ControlVars.VqSat);
}
