/*
   Copyright (c) 2015 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
*/

#include "main.h"
#include "control.h"
#include "pu.h"
#include "pwm.h"
#include "hall.h"
#include "calib.h"
#include "macros.h"
#include "tm_stm32f4_adc.h"

//structure for control variables
struct CONTROL_VARS ControlVars;

//instance a global hall effect driver
HALL3 Hall1 = HALL3_DEFAULTS;

//structure for Analog Inputs
struct ANALOG_INPUTS AIn;

//debugging
uint16_t ControlStepCtr;

//instance a PWM driver instance
static PWMGEN pwm1 = PWMGEN_DEFAULTS;

#define ANGLE_60DEG 0x100000L

int Hall0Lookup[7] =
{ 0,     // 0
      0, // 1
      4, // 2
      5, // 3
      2, // 4
      1, // 5
      3  // 6
      };

struct PI_RECORD
{
   int16_t Kp;      // proportional gain (Q_KP)
   int16_t Ki;      // integral gain (Q_KI)
   int16_t Kpi;     // sum of Kp+Ki (Q_KP)
   int16_t Kc;      // anti-windup gain Ki/(Ki+Kp) (Q_KI)
   int32_t integ32; // integrator
   int32_t out32;
   int16_t out;
} PIRec;

/*
 * Reset PI Record
 */
void resetPI(struct PI_RECORD *aPi)
{
   aPi->Kpi = aPi->Kp + (aPi->Ki >> 4);
   aPi->Kc = Q_DIV(aPi->Ki, aPi->Kpi, 10);
   aPi->integ32 = 0;
}

/*
 * Update of proportional part
 */
void updatePIFirst(struct PI_RECORD *aPi, int16_t aError)
{
   aPi->out32 = (aPi->integ32 >> (Q_KI - Q_KP)) + (int32_t) aPi->Kpi * (int32_t) aError;
   aPi->out = (int16_t) (aPi->out32 >> Q_KP);
}

/*
 * Update of integrative part and anti-windup
 */
void updatePISecond(struct PI_RECORD *aPi, int16_t aError, int16_t aOutlim)
{
   aPi->integ32 = aPi->integ32 + (int32_t) aPi->Ki * (int32_t) aError;               // update integrator
   aPi->integ32 = aPi->integ32 - (int32_t) aPi->Kc * (int32_t) (aPi->out - aOutlim); // anti-windup
}

/*
 * Initializes control task.
 */
void initControl(void)
{
	 //Set PWM frequency
   ControlVars.Fs = 10000;

   //Set ControlVars Parameters
   ControlVars.MPosAccuCtr = -1;
   ControlVars.ClosedFlag = FALSE;
   ControlVars.PosCtr = 0;
   ControlVars.Pos = 0;

   // Initialize Hall module
   Hall1.DebounceAmount = 0;
   Hall1.Revolutions = -3;
   initHall3(&Hall1);

	 //Initialize ADC1 on channel 0 -> pin PA4
	 TM_ADC_Init(ADC1, ADC_Channel_4);
	 AIn.Idc = 0;
	 
	 //Initialize Timer modules
   pwm1.DutyFunc = 0;
   initPWM(&pwm1);
	
   //Initialize control system
	 initCalib();
   stopControl();
   resetControl();
}

/**
 * Called when stopping control, disabling actuators.
 */
void stopControl(void)
{
   // disable actuators -> red LED off
   TM_DISCO_LedOff(LED_RED);
	 //Reset MOE
	 TIM1->BDTR &= 0x7FFF;
	 // "idle" PWM generator
   pwm1.CmtnPointer = 0;
   pwm1.DutyFunc = 0;
   updatePWM(&pwm1);
}

/**
 * Enable actuators.
 */
void startControl(void)
{
   resetControl();

#ifndef KEEP_POWERSTAGE_OFF
   // enable actuators -> red LED on
	 TM_DISCO_LedOn(LED_RED);
	 //Set MOE
	 TIM1->BDTR |= 0x8000;	 
#endif
}

/**
 * Called when control starts, initialize filters, regulators here.
 */
void resetControl(void)
{
   ControlStepCtr = 0;

   ControlVars.Vmax = Calib.Vmax;
   ControlVars.Vset = 0;
   PIRec.Kp = Calib.Kp;
   PIRec.Ki = Calib.Ki;

   resetPI(&PIRec);
}

/**
 * Process analog and digital inputs
 */
void readInputs()
{
	int16_t deltaPos0;
	
   //current sensor offset calibration (simple averaging)
   if (AIn.AccuCtr < 256)
   {

      AIn.IdcAccu += ADC_GetConversionValue(ADC1);
      AIn.AccuCtr++;
      if (AIn.AccuCtr == 256)
      {
         AIn.IdcOffset = (int) (AIn.IdcAccu >> 8);//Average of first 256 measurements
      }
   }
   
	 //***************************************************************************************************************
   //DC Current is a Probed Variable -> write access needs to be replaced by SET_OPROBE

   //Original Code:

   //AIn.Idc = AdcResult.ADCRESULT6 - AIn.IdcOffset;
   //AIn.Idc = Q_MUL(AIn.Idc, Calib.currentGain, Q_IV);

   //Replacement:

   SET_OPROBE(AIn.Idc, ADC_GetConversionValue(ADC1) - AIn.IdcOffset);
   SET_OPROBE(AIn.Idc, Q_MUL(AIn.Idc, Calib.currentGain, Q_IV));
	 
	 //***************************************************************************************************************

	 //update position measurement
   determineHALL3State(&Hall1);

   //update speed measurement
   if ((Hall1.HallGpio > 0) && (Hall1.HallGpio < 7))
   {
      if (ControlVars.MPosAccuCtr < 0)
      {
         // speed measurement needs to be initialized
         ControlVars.MPos0 = Hall0Lookup[Hall1.HallGpio];
         ControlVars.MPos0Last = ControlVars.MPos0;
         ControlVars.MPosAccumulated = 0;
         ControlVars.MPosAccuCtr = 0;
      }
      else
      {
         ControlVars.MPos0 = Hall0Lookup[Hall1.HallGpio];
         deltaPos0 = ControlVars.MPos0 - ControlVars.MPos0Last;
         if (deltaPos0 >= 5)
         {
            deltaPos0 -= 6;
         }
         else if (deltaPos0 <= -5)
         {
            deltaPos0 += 6;
         }
         ControlVars.MPosAccumulated += deltaPos0;
         ControlVars.MPos0Last = ControlVars.MPos0;
         ControlVars.MPosAccuCtr++;
         if (ControlVars.MPosAccuCtr == 100)
         {
            ControlVars.MPosAccuCtr = 0;
            ControlVars.MSpeed =
                  Q_MUL(ControlVars.MPosAccumulated, Calib.MSpeedScale, 0);
            ControlVars.MPosAccumulated = 0;

            // basic overspeed protection
            if (ControlVars.MSpeed > Calib.maxMSpeed)
            {
               stopControl();
            }
         }
      }
   }
}

/**
 * Perform a control Step
 */
void controlStep()
{
	 ControlStepCtr++;	 

   //***************************************************************************************************************
   //Current Setpoint is a Probed Variable -> write access needs to be replaced by SET_OPROBE

   //Original Code:

   //ControlVars.Iset = Calib.Iset

   //Replacement:
	 SET_OPROBE(ControlVars.Iset, Calib.Iset);

   //***************************************************************************************************************
	
 	 //determine actual current error
	 ControlVars.Ierr = ControlVars.Iset - AIn.Idc;	 

   //run 1st half of PI, which results in new output
   updatePIFirst(&PIRec, ControlVars.Ierr);
   //saturate output
   if (PIRec.out > ControlVars.Vmax)
   {
			ControlVars.Vset = ControlVars.Vmax;			
   }
   else if (PIRec.out < 0)
   {
			ControlVars.Vset = 0;  
   }
   else
   {
			ControlVars.Vset = PIRec.out;     
   }

   //calculate PWM duty cycle
   ControlVars.DutyFunc = Q_MUL(ControlVars.Vset, Calib.PWMGain, (10+14-15)); // Q10*Q14 -> Q15

   //update motor position and commutation step
   ControlVars.MPos0Angle = Calib.commAngle0;//Cmtn Pointer Offset
   ControlVars.CmtnPointer = ControlVars.MPos0 + ControlVars.MPos0Angle;
   if (ControlVars.CmtnPointer < 0)
   {
      ControlVars.CmtnPointer += 6;
   }
   else if (ControlVars.CmtnPointer > 5)
   {
      ControlVars.CmtnPointer -= 6;
   }

   // update modulator
	 pwm1.CmtnPointer = ControlVars.CmtnPointer;
   pwm1.DutyFunc = ControlVars.DutyFunc;
   updatePWM(&pwm1);

   // run second half of PI, which updates integrator  
	 updatePISecond(&PIRec, ControlVars.Ierr, ControlVars.Vset);	 
}
