/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

// function prototypes
void DeviceInit(void);
void MemCopy(Uint16 *SourceAddr, Uint16 *SourceEndAddr, Uint16 *DestAddr);
void InitFlash();

// linker addresses, needed to copy code from flash to ram
extern Uint16 RamfuncsLoadStart, RamfuncsLoadEnd, RamfuncsRunStart;

// LED blink period/2
#define LED_BLINK_PERIOD_2 (500 * TICKS_PER_MS)

// timer variables
Uint16 LEDTimer = 0; // for LED blinking

struct CTRL_DISPATCH_VARS CtrlDispatchVars;

interrupt void ControlDispatcher(void)
{
	CtrlDispatchVars.interruptNesting++;

	PIL_beginInterruptCall();

	// Reinitialize for next ADC sequence
	AdcRegs.ADCTRL2.bit.RST_SEQ1 = 1;         // Reset SEQ1
	AdcRegs.ADCST.bit.INT_SEQ1_CLR = 1;       // Clear INT SEQ1 bit
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;   // Acknowledge interrupt to PIE
	IER |= M_INT1;

	LEDTimer++;
	// re-enable interrupt to allow sub-task to be interrupted
	EINT;

	ControlTask1(); // this task is always dispatched

	bool_t dispatchTask2 = (CtrlDispatchVars.task2DispatchTimer == 0);

	CtrlDispatchVars.task2DispatchTimer++;
	if(CtrlDispatchVars.task2DispatchTimer >= TASK2_PERIOD)
	{
		CtrlDispatchVars.task2DispatchTimer = 0;
	}

	if(dispatchTask2)
	{
		CtrlDispatchVars.task2Active = TRUE;
		ControlTask2();
		CtrlDispatchVars.task2Active = FALSE;
	}

	DINT; // // TI expects interrupts are disabled before entering I$$REST
	CtrlDispatchVars.interruptNesting--;
}

void ResetControlDispatcher()
{
	CtrlDispatchVars.task2DispatchTimer = 0;
}

void InitControlDispatcher()
{
	// Configure triggering of ADC by EPWM
	EPwm1Regs.ETSEL.bit.SOCAEN = 1;        // Enable SOC on A group
	EPwm1Regs.ETSEL.bit.SOCASEL = 6;       // Select SOC from from CPMB on upcount
	EPwm1Regs.ETPS.bit.SOCAPRD = 1;        // Generate pulse on 1st event
	AdcRegs.ADCTRL2.bit.EPWM_SOCA_SEQ1 = 1;// Enable SOCA from ePWM to start SEQ1

	// Trigger control-interrupt when ADC is finished
	AdcRegs.ADCTRL2.bit.INT_ENA_SEQ1 = 1;  // Enable SEQ1 interrupt (every EOS)
	EALLOW;
	PieVectTable.ADCINT = &ControlDispatcher;
	EDIS;
	PieCtrlRegs.PIEIER1.bit.INTx6 = 1;
	IER |= M_INT1;

	CtrlDispatchVars.interruptNesting = 0;
	CtrlDispatchVars.task2Active = FALSE;
	ResetControlDispatcher();
}

void BlinkLed(void){
	if(LEDTimer > LED_BLINK_PERIOD_2){
#ifdef LED_ON_33
		GpioDataRegs.GPBTOGGLE.bit.GPIO33 = 1;
#endif
#ifdef LED_ON_32
		GpioDataRegs.GPBTOGGLE.bit.GPIO32 = 1;
#endif
#ifdef LED_ON_34
		GpioDataRegs.GPBTOGGLE.bit.GPIO34 = 1;
#endif
		LEDTimer = 0;
	}
}

void main(void)
{
	DeviceInit();

	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

	InitFlash();

	EALLOW;
#ifdef LED_ON_33
	GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 0;
	GpioCtrlRegs.GPBDIR.bit.GPIO33 = 1;
#endif
#ifdef LED_ON_32
	GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 0;
	GpioCtrlRegs.GPBDIR.bit.GPIO32 = 1;
#endif
#ifdef LED_ON_34
	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;
	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 1;
#endif	
	EDIS;

	DINT;
	IER = 0x0000;
	IFR = 0x0000;

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;      // Stop all the TB clocks
	EDIS;

	// configure serial line
	SCIInit(ONE_STOPBIT+ NO_PARITY + EIGHT_BITS, BAUD_RATE, LSPCLK_HZ);
	SCIWriteString("\n\rRS-232 Initialized.\n\r");

	// initialize PIL framework
	PIL_init();
	PIL_setLinkParams(PIL_GUID_PTR, (PIL_CommCallbackPtr_t)SCIPoll);
	PIL_setCtrlCallback((PIL_CtrlCallbackPtr_t)PilCallback);

	PilInitOverrideProbes();
	PilInitCalibrations();

	// init tasks
	InitControl();
	InitControlDispatcher();

	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;         // Start all the timers synced
	EDIS;

	// get ready for PIL
	PIL_requestReadyMode();

	// main-loop
	for(;;)
	{
		BlinkLed();
		PIL_backgroundCall();
		ControlBackground();
	}
}
