/*
   Copyright (c) 2014 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

// function prototypes
void InitFlash();

// linker addresses, needed to copy code from flash to ram
extern Uint16 RamfuncsLoadStart, RamfuncsLoadSize, RamfuncsRunStart;

// LED blink period/2
#define LED_BLINK_PERIOD_2 (500 * TICKS_PER_MS)

uint16_t LEDTimer;

struct CTRL_DISPATCH_VARS CtrlDispatchVars;

__interrupt void ControlDispatcher(void)
{
	CtrlDispatchVars.interruptNesting++;

	PIL_beginInterruptCall();

#if defined(USE_CPU_TIMER_INT)
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
#elif defined(USE_ADC_INT)
	AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;	// Clear ADCINT1 flag reinitialize for next SOC
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1; // Acknowledge interrupt to PIE
	IER |= M_INT1;

#else
	EPwm1Regs.ETCLR.bit.INT = 1;  // Clear INT flag for this timer
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP3; // Acknowledge interrupt to PIE
	IER |= M_INT3;
#endif

	LEDTimer++;
	// re-enable interrupt to allow sub-task to be interrupted
	EINT;

	ControlTask1(); // this task is always dispatched

	bool_t dispatchTask2 = (CtrlDispatchVars.task2DispatchTimer == 0);

	CtrlDispatchVars.task2DispatchTimer++;
	if(CtrlDispatchVars.task2DispatchTimer >= TASK2_PERIOD)
	{
		CtrlDispatchVars.task2DispatchTimer = 0;
	}

	if(dispatchTask2)
	{
		CtrlDispatchVars.task2Active = TRUE;
		ControlTask2();
		CtrlDispatchVars.task2Active = FALSE;
	}

	DINT; // // TI expects interrupts are disabled before entering I$$REST
	CtrlDispatchVars.interruptNesting--;
}

void ResetControlDispatcher()
{
	CtrlDispatchVars.task2DispatchTimer = 0;
}

void InitControlDispatcher()
{
#if defined(USE_CPU_TIMER_INT)
	EALLOW;
	PieVectTable.TIMER0_INT = &ControlDispatcher;
	EDIS;

	InitCpuTimers();

	ConfigCpuTimer(&CpuTimer0, SYSCLK_MHZ, 1000000L/PWM_HZ);
	CpuTimer0Regs.TCR.all = 0x4000; // Use write-only instruction to set TSS bit = 0
	IER |= M_INT1;

	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;

#elif defined(USE_ADC_INT)
	EALLOW;

	EPwm1Regs.ETSEL.bit.SOCAEN	= 1;		// Enable SOC on A group
	EPwm1Regs.ETSEL.bit.SOCASEL	= 2;		// Select SOC trigger on ePWM period
	EPwm1Regs.ETPS.bit.SOCAPRD 	= 1;		// Generate pulse on 1st event

	AdcaRegs.ADCINTSEL1N2.bit.INT1CONT  = 0;	// Disable ADCINT1 Continuous mode
	AdcaRegs.ADCINTSEL1N2.bit.INT1SEL 	= 0;    // setup EOC2 to trigger ADCINT1 (after Iu, Iv, Idc)
	AdcaRegs.ADCCTL1.bit.INTPULSEPOS	= 1;	// ADCINT1 trips after AdcResults latch
	AdcaRegs.ADCINTSEL1N2.bit.INT1E     = 1;	// Enable ADCINT1
	EDIS;

	EALLOW;
	PieVectTable.ADCA1_INT = &ControlDispatcher;
	EDIS;
	PieCtrlRegs.PIEIER1.bit.INTx1 = 1;
	IER |= M_INT1;

#else
	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_PRD; // Select INT on period event
	EPwm1Regs.ETSEL.bit.INTEN = 1;           // Enable INT
	EPwm1Regs.ETPS.bit.INTPRD = ET_1ST; 	 // Generate interrupt on 1st event

	EALLOW;
	PieVectTable.EPWM1_INT = &ControlDispatcher;
	EDIS;
	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;
	IER |= M_INT3;
#endif

	CtrlDispatchVars.interruptNesting = 0;
	CtrlDispatchVars.task2Active = FALSE;
	ResetControlDispatcher();
}

void BlinkLed(void)
{
	static bool_t blink;

	if(LEDTimer > LED_BLINK_PERIOD_2)
	{
		if(blink){
			GPIO_WritePin(CPU1_LED_GPIO, 1);
			blink = FALSE;
		} else {
			GPIO_WritePin(CPU1_LED_GPIO, 0);
			blink = TRUE;
		}
		LEDTimer = 0;
	}
}

void main(void)
{
#ifdef _FLASH
	memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);
#endif
	// Step 1. Initialize System Control:
	// PLL, WatchDog, enable Peripheral Clocks
	// This example function is found in the F2837xD_SysCtrl.c file.
	InitSysCtrl();

	// Call Flash Initialization to setup flash waitstates
	// This function must reside in RAM
#ifdef _FLASH
	InitFlash();
#endif

	InitGpio();
	// Configure IO for LED blinking
	GPIO_SetupPinMux(CPU1_LED_GPIO, GPIO_MUX_CPU1, 0);
	GPIO_SetupPinOptions(CPU1_LED_GPIO, GPIO_OUTPUT, GPIO_PUSHPULL);

	DINT;

	InitPieCtrl();

	IER = 0x0000;
	IFR = 0x0000;

	InitPieVectTable();

	EALLOW;
	CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 0;      // Stop all the TB clocks
	EDIS;

	// configure serial line
	SCIInit(ONE_STOPBIT+ NO_PARITY + EIGHT_BITS, BAUD_RATE, LSPCLK_HZ);
	SCIWriteString("\n\rRS-232 Initialized.\n\r");

	// initialize PIL framework
	PIL_init();
	PIL_setLinkParams((unsigned char*)&PIL_D_Guid[0], (PIL_CommCallbackPtr_t)SCIPoll);
	PIL_setCtrlCallback((PIL_CtrlCallbackPtr_t)PilCallback);

	PilInitOverrideProbes();
	PilInitCalibrations();

	InitControlDispatcher();
	InitControl();

	// Enable global Interrupts and higher priority real-time debug events:
	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM

	EALLOW;
	CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;         // Start all the timers synced
	EDIS;

	// get ready for PIL
	PIL_requestReadyMode();

	// main-loop
	for(;;) {
		BlinkLed();
		PIL_backgroundCall();
		ControlBackground();
	}
}
